Repository navigation
CUDA execution for external patches #7376
New issue
Have a question about this project? Sign up for a free GitHub account to open an issue and contact its maintainers and the community.
By clicking “Sign up for GitHub”, you agree to our terms of service and privacy statement. We’ll occasionally send you account related emails.
Already on GitHub? Sign in to your account
Closed
Closed
Changes from all commits
Commits
File filter
Filter by extension
Conversations
Failed to load comments.
Loading
Jump to
Jump to file
Failed to load files.
Loading
Diff view
Diff view
There are no files selected for viewing
This file contains hidden or bidirectional Unicode text that may be interpreted or compiled differently than what appears below. To review, open the file in an editor that reveals hidden Unicode characters.
Learn more about bidirectional Unicode characters
This file contains hidden or bidirectional Unicode text that may be interpreted or compiled differently than what appears below. To review, open the file in an editor that reveals hidden Unicode characters.
Learn more about bidirectional Unicode characters
This file contains hidden or bidirectional Unicode text that may be interpreted or compiled differently than what appears below. To review, open the file in an editor that reveals hidden Unicode characters.
Learn more about bidirectional Unicode characters
This file contains hidden or bidirectional Unicode text that may be interpreted or compiled differently than what appears below. To review, open the file in an editor that reveals hidden Unicode characters.
Learn more about bidirectional Unicode characters
| Original file line number | Diff line number | Diff line change |
|---|---|---|
| @@ -0,0 +1,65 @@ | ||
| // SPDX-License-Identifier: Apache-2.0 | ||
| // SPDX-FileCopyrightText: Copyright the Vortex contributors | ||
|
|
||
| #include "types.cuh" | ||
|
|
||
| /// Apply patches to an output array using the transposed Patched array format. | ||
| /// | ||
| /// This kernel uses a thread-per-lane model where each thread is assigned to | ||
| /// one (chunk, lane) slot and applies all patches in that slot. | ||
| template <typename ValueT> | ||
| __device__ void patched(ValueT *const output, | ||
| const uint32_t *const lane_offsets, | ||
| const uint16_t *const patch_indices, | ||
| const ValueT *const patch_values, | ||
| uint32_t n_lanes, | ||
| uint32_t total_lane_slots, | ||
| uint64_t offset, | ||
| uint64_t len) { | ||
| const uint32_t lane_slot = blockIdx.x * blockDim.x + threadIdx.x; | ||
|
|
||
| // Early return if this thread is beyond the number of lane slots | ||
| if (lane_slot >= total_lane_slots) { | ||
| return; | ||
| } | ||
|
|
||
| // Determine which chunk this lane slot belongs to | ||
| const uint32_t chunk = lane_slot / n_lanes; | ||
|
|
||
| // Get the range of patches for this lane slot | ||
| const uint32_t start = lane_offsets[lane_slot]; | ||
| const uint32_t stop = lane_offsets[lane_slot + 1]; | ||
|
|
||
| // Apply all patches in this lane | ||
| for (uint32_t p = start; p < stop; p++) { | ||
| // Get within-chunk index and compute global position | ||
| const uint16_t within_chunk_idx = patch_indices[p]; | ||
| const uint64_t global_idx = static_cast<uint64_t>(chunk) * 1024 + within_chunk_idx; | ||
|
|
||
| // Check bounds (for sliced arrays) | ||
| if (global_idx < offset) { | ||
| continue; | ||
| } | ||
|
|
||
| if (global_idx >= offset + len) { | ||
| break; | ||
| } | ||
|
|
||
| output[global_idx - offset] = patch_values[p]; | ||
| } | ||
| } | ||
|
|
||
| #define GENERATE_PATCHED_KERNEL(value_suffix, ValueT) \ | ||
| extern "C" __global__ void patched_##value_suffix(ValueT *const output, \ | ||
| const uint32_t *const lane_offsets, \ | ||
| const uint16_t *const patch_indices, \ | ||
| const ValueT *const patch_values, \ | ||
| uint32_t n_lanes, \ | ||
| uint32_t total_lane_slots, \ | ||
| uint64_t offset, \ | ||
| uint64_t len) { \ | ||
| patched(output, lane_offsets, patch_indices, patch_values, n_lanes, total_lane_slots, offset, len); \ | ||
| } | ||
|
|
||
| // Generate for all native SIMD ptypes | ||
| FOR_EACH_NATIVE_SIMD_PTYPE(GENERATE_PATCHED_KERNEL) | ||
This file contains hidden or bidirectional Unicode text that may be interpreted or compiled differently than what appears below. To review, open the file in an editor that reveals hidden Unicode characters.
Learn more about bidirectional Unicode characters
This file contains hidden or bidirectional Unicode text that may be interpreted or compiled differently than what appears below. To review, open the file in an editor that reveals hidden Unicode characters.
Learn more about bidirectional Unicode characters
This file contains hidden or bidirectional Unicode text that may be interpreted or compiled differently than what appears below. To review, open the file in an editor that reveals hidden Unicode characters.
Learn more about bidirectional Unicode characters
Oops, something went wrong.
Add this suggestion to a batch that can be applied as a single commit.
This suggestion is invalid because no changes were made to the code.
Suggestions cannot be applied while the pull request is closed.
Suggestions cannot be applied while viewing a subset of changes.
Only one suggestion per line can be applied in a batch.
Add this suggestion to a batch that can be applied as a single commit.
Applying suggestions on deleted lines is not supported.
You must change the existing code in this line in order to create a valid suggestion.
Outdated suggestions cannot be applied.
This suggestion has been applied or marked resolved.
Suggestions cannot be applied from pending reviews.
Suggestions cannot be applied on multi-line comments.
Suggestions cannot be applied while the pull request is queued to merge.
Suggestion cannot be applied right now. Please check back later.
There was a problem hiding this comment.
Choose a reason for hiding this comment
The reason will be displayed to describe this comment to others. Learn more.
Nice! Ideally we could move to a world where we drop the other
patches.cueventually and only operate on transposed patches on the GPU, maybe.